﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// Import potrebného menného priestoru. 
using System.Threading;

namespace ParallelReduction
{
    class Program
    {
        static void Main(string[] args)
        {
            int total = 0;
            // Generovanie dátového poľa s číselnou sekvenciou.
            int[] numbers = Enumerable.Range(0, 10).Select(x => x * x * x).ToArray();
            // Výpočet kontrolného súčtu.
            int totalCheck = Enumerable.Range(0, 10).Select(x => x * x * x).Sum();

            // Paralelná redukcia sa uskutočňuje v paralelnom cykle.
            Parallel.For(0, numbers.Length, () => 0, (x, loopState, locAcc) =>
            {
                locAcc += numbers[x];
                return locAcc;
            },
            (locAcc) =>
            {
                // Redukovaná hodnota smeruje do globálneho akumulátora. 
                // Keďže ten je zdieľaný, jeho aktualizáciu realizujeme atomicky.
                Interlocked.Add(ref total, locAcc);
            });

            // Výpis číselnej sekvencie z dátového poľa.
            Console.WriteLine("Sequence of numbers: ");
            foreach (var number in numbers)
            {
                Console.WriteLine(number);
            }
            // Verifikácia korektnosti paralelného súčtu.
            if (total == totalCheck)
            {
                Console.WriteLine("Total sum: {0}.", total);
            }
            else
            {
                Console.WriteLine("Computation error occured.");
            }
        }
    }
}
