﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// Import dodatočných menných priestorov.
using System.Diagnostics;
using System.Drawing;

namespace Futures
{
    class Program
    {
        // Cieľová transformačná metóda na inverziu rastra.
        static long GT_Inversion(string sourceBitmap, string targetBitmap,
        bool verboseMode = false)
        {
            Bitmap bitmap = new Bitmap(sourceBitmap);
            Color oldColorPixel, newColorPixel;
            Stopwatch sw = new Stopwatch();

            int bitmapHeight = bitmap.Height;
            int bitmapWidth = bitmap.Width;

            sw.Start();
            if (verboseMode)
            {
                Console.WriteLine("Bitmap {0} inversion started: [{1}].",
                    sourceBitmap, DateTime.Now);
            }

            // Transformácia 2D rastra. 
            for (int i = 0; i < bitmapWidth; i++)
            {
                for (int j = 0; j < bitmapHeight; j++)
                {
                    oldColorPixel = bitmap.GetPixel(i, j);
                    newColorPixel = Color.FromArgb(
                        255 - oldColorPixel.R,
                        255 - oldColorPixel.G,
                        255 - oldColorPixel.B);
                    bitmap.SetPixel(i, j, newColorPixel);
                }
            }
            sw.Stop();

            if (verboseMode)
            {
                Console.WriteLine("Bitmap {0} inversion finished: [{1}].",
                    targetBitmap, DateTime.Now);
            }
            // Uloženie transformovaného rastra do nového súboru.
            bitmap.Save(targetBitmap);
            bitmap.Dispose();
            return sw.ElapsedMilliseconds;
        }

        // Cieľová transformačná metóda na sivotónovanie rastra.
        static long GT_Grayscale(string sourceBitmap, string targetBitmap,
        bool verboseMode = false)
        {
            Bitmap bitmap = new Bitmap(sourceBitmap);

            Color oldColorPixel, newColorPixel;
            int intensity;
            Stopwatch sw = new Stopwatch();

            int bitmapHeight = bitmap.Height;
            int bitmapWidth = bitmap.Width;

            sw.Start();
            if (verboseMode)
            {
                Console.WriteLine("Bitmap {0} tranformation to grayscale started: [{1}].",
                   sourceBitmap, DateTime.Now);
            }
            // Transformácia 2D rastra. 
            for (int i = 0; i < bitmapWidth; i++)
            {
                for (int j = 0; j < bitmapHeight; j++)
                {
                    oldColorPixel = bitmap.GetPixel(i, j);
                    intensity = (int)(0.299 * oldColorPixel.R +
                        0.587 * oldColorPixel.G + 0.114 * oldColorPixel.B);
                    newColorPixel = Color.FromArgb(intensity, intensity, intensity);
                    bitmap.SetPixel(i, j, newColorPixel);
                }
            }
            sw.Stop();

            if (verboseMode)
            {
                Console.WriteLine("Bitmap {0} transformation to grayscale finished: [{1}].",
                    targetBitmap, DateTime.Now);
            }
            // Uloženie transformovaného rastra do nového súboru.
            bitmap.Save(targetBitmap);
            bitmap.Dispose();
            return sw.ElapsedMilliseconds;
        }

        // Hlavná metóda paralelného programu.
        static void Main(string[] args)
        {
            string currentDirectory = Environment.CurrentDirectory;
            // Konštrukcia kolekcie futurít. 
            Task<long>[] transformationTasks = new Task<long>[] 
        {
            new Task<long>
            ( 
                () => GT_Inversion(currentDirectory + @"\Pictures\bitmap1.jpg", 
                    currentDirectory + @"\Pictures\bitmap1_GT.jpg", true)
            ), 
            new Task<long>
            ( 
                () => GT_Inversion(currentDirectory + @"\Pictures\bitmap2.jpg", 
                    currentDirectory + @"\Pictures\bitmap2_GT.jpg", true)
            ),
            new Task<long>
            ( 
                () => GT_Inversion(currentDirectory + @"\Pictures\bitmap3.jpg", 
                    currentDirectory + @"\Pictures\bitmap3_GT.jpg", true)
            ),
            new Task<long>
            ( 
                () => GT_Inversion(currentDirectory + @"\Pictures\bitmap4.jpg", 
                    currentDirectory + @"\Pictures\bitmap4_GT.jpg", true)
            ),
        };

            // Naplánovanie futurít na spracovanie. 
            foreach (var task in transformationTasks)
            {
                task.Start();
            }

            // Stanovenie explicitnej bariéry v paralelnom kóde. 
            Task.WaitAll(transformationTasks);

            // Dolovanie výsledkov z futurít.
            long inversionTime1 = transformationTasks[0].Result;
            long inversionTime2 = transformationTasks[1].Result;
            long grayscaleTime1 = transformationTasks[2].Result;
            long grayscaleTime2 = transformationTasks[3].Result;

            // Zobrazenie výsledkov futurít na výstupe.
            Console.WriteLine("Inversion time 1: {0} ms.", inversionTime1);
            Console.WriteLine("Inversion time 2: {0} ms.", inversionTime2);
            Console.WriteLine("Grayscale time 1: {0} ms.", grayscaleTime1);
            Console.WriteLine("Grayscale time 2: {0} ms.", grayscaleTime2);
            Console.ReadLine();
        }
    }
}
