﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// Importovanie dodatočných menných priestorov.
using System.Diagnostics;
using System.Collections.Concurrent;


namespace ParallelProducersConsumer
{
    // Deklarácia podpornej triedy, z ktorej vzniknú objekty s dátami a časovou značkou.
    internal class TimeData
    {
        public double Data { get; set; }
        public DateTime TimeStamp { get; set; }
    }

    class Program
    {
        // Hlavná metóda paralelného programu. 
        static void Main(string[] args)
        {
            // Vytvorenie konkurenčného frontu ako objektového repozitára.
            ConcurrentQueue<TimeData> concQueue = new ConcurrentQueue<TimeData>();

            // Stanovenie počtu paralelných producentov. 
            const int producersCount = 10;
            // Určenie počtu pridávaných objektov do frontu na jedného producenta.
            const int itemsCount = 1000000;
            Stopwatch sw = new Stopwatch();

            // Vytvorenie kolekcie paralelných producentov.
            Task[] producers = new Task[producersCount];
            Random generator = new Random();

            Console.WriteLine("Starting producers and queuing the concQueue [{0}].",
                DateTime.Now);

            sw.Start();
            // Definovanie logiky paralelných producentov a ich naštartovanie. 
            Parallel.For(0, producersCount, i =>
            {
                producers[i] = new Task(() =>
                {
                    Parallel.For(0, itemsCount, p =>
                    {
                        concQueue.Enqueue(new TimeData()
                        {
                            Data = generator.Next(1, 11),
                            TimeStamp = DateTime.Now
                        });
                    });
                });
                producers[i].Start();
            });

            // Čakanie na dokončenie práce paralelných producentov. 
            Task.WaitAll(producers);
            sw.Stop();

            Console.WriteLine("Total time for queuing: {0} ms.", sw.ElapsedMilliseconds);

            // Vytvorenie konzumenta. 
            Task consumer = new Task(() =>
            {
                int dequeuedItemsCount;
                while (!concQueue.IsEmpty)
                {
                    dequeuedItemsCount = 0;
                    TimeData result;
                    concQueue.TryDequeue(out result);
                    dequeuedItemsCount++;
                }
            });

            Console.WriteLine("Starting consumer and dequeuing the concQueue [{0}].",
                DateTime.Now);
            sw.Restart();
            // Aktivácia konzumenta. 
            consumer.Start();
            // Čakanie na dokončenie práce konzumenta.
            consumer.Wait();
            sw.Stop();

            Console.WriteLine("Total time for dequeuing: {0} ms.", sw.ElapsedMilliseconds);
            Console.WriteLine("Is ConcQueue empty? {0}.", concQueue.IsEmpty ? "Yes" : "No");
            Console.ReadLine();
        }
    }

}
