﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

// Import dodatočných menných priestorov.
using System.Diagnostics;
using System.Collections.Concurrent;

namespace ParallelMetrics
{
    // Deklarácia podpornej triedy.
    internal class DataRecord
    {
        public DateTime Time { get; set; }
        public double Data { get; set; }
    }

    class Program
    {
        static void Main(string[] args)
        {
            // Detekcia logických mikroprocesorov počítača.
            int logicalCPUs = Environment.ProcessorCount;
            int tasksCount = logicalCPUs;
            // Tvorba kolekcie futurít.
            Task<long>[] tasks = new Task<long>[tasksCount];
            // Inštanciácia konkurenčného slovníka.
            ConcurrentDictionary<long, DataRecord> concDictionary =
                new ConcurrentDictionary<long, DataRecord>();
            Console.WriteLine("Metrics computation started [{0}].", DateTime.Now);

            // Konštrukcia futurít a programovanie ich logiky pri inicializácii 
            // konkurenčného slovníka.
            Parallel.For(0, tasksCount, i =>
            {
                tasks[i] = Task.Factory.StartNew<long>(
                    new Func<long>(
                        () =>
                        {
                            Stopwatch sw = new Stopwatch();
                            sw.Start();
                            Parallel.For(0, (int)Math.Pow(10, 6), item =>
                            {
                                concDictionary.TryAdd((new Random()).Next(i, i * 100),
                                    new DataRecord() { Data = i, Time = DateTime.Now });
                            });
                            sw.Stop();
                            return sw.ElapsedMilliseconds;
                        }));
            });
            // Stanovenie explicitnej bariéry pre všetky futurity z kolekcie.
            Task.WaitAll(tasks);

            Console.WriteLine("Metrics computation finished [{0}].", DateTime.Now);
            Console.WriteLine("Time statistics:");

            // Iteratívne získanie výsledkov z futurít. 
            foreach (var task in tasks)
            {
                Console.WriteLine("Task (ID: {0}) -> Time: {1} ms.", task.Id, task.Result);
            }
        }
    }
}
